@@ -6,6 +6,8 @@ from AOT_biomaps.AOT_Acoustic.StructuredWave import StructuredWave
 from abc import ABC, abstractmethod
 import os
 import numpy as np
+import torch
+import torch.nn.functional as F
 from tqdm import trange
 from datetime import datetime
 import matplotlib.pyplot as plt
@@ -69,6 +71,39 @@ class Experiment(ABC):
                 raise ValueError(f"Field {field.getName_field()} has an invalid shape: {field.field.shape}. Expected shape to be at least ({max_sample},).")
             self.AcousticFields[i].field = field.field[min_sample:max_sample, :, :]
 
+    def reduceDims(self, mode='avg'):
+        """
+        Réduit les dimensions T, Z, X d'une matrice (T, Z, X, N) par 2 en utilisant une convolution.
+        Args:
+            matrix (torch.Tensor ou np.ndarray): Matrice d'entrée de forme (T, Z, X, N).
+            mode (str): 'avg' (moyenne) ou 'max' (max-pooling).
+        Returns:
+            torch.Tensor: Matrice réduite de forme (T//2, Z//2, X//2, N).
+        """
+        for i, field in enumerate(self.AcousticFields):
+            # Conversion en tenseur PyTorch si nécessaire
+            if isinstance(field.field, np.ndarray):
+                field.field = torch.from_numpy(field.field)
+
+            # Permutation des axes pour adapter à conv3d: (N, 1, T, Z, X)
+            x = field.field.permute(3, 0, 1, 2).unsqueeze(1)  # Ajoute un canal fictif
+
+            # Réduction par convolution 3D
+            if mode == 'avg':
+                x_down = F.avg_pool3d(x, kernel_size=(2, 2, 2), stride=(2, 2, 2))
+            elif mode == 'max':
+                x_down = F.max_pool3d(x, kernel_size=(2, 2, 2), stride=(2, 2, 2))
+            else:
+                raise ValueError("Mode invalide. Utilisez 'avg' ou 'max'.")
+
+            # Retour à l'ordre original (T//2, Z//2, X//2, N)
+            self.AcousticFields[i].field = x_down.squeeze(1).permute(1, 2, 3, 0)
+        
+        self.params.acoustic['f_saving'] /= 2
+        self.params.general['dx'] *= 2
+        self.params.general['dz'] *= 2
+        self.params.general['dy'] *= 2
+
     def saveAcousticFields(self, save_directory):
         progress_bar = trange(len(self.AcousticFields), desc="Saving Acoustic Fields")
         for i in progress_bar:
